<?php

/**
*
* @package PM Spy
* @copyright (c) 2015 www.phpbb-work.ru
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

namespace shredder\pm_spy\acp;

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/**
* @package acp
*/
class pm_spy_module
{
	/** @var \phpbb\config\config */
	protected $config;

	/** @var \phpbb\db\driver\driver_interface */
	protected $db;

	/** @var \phpbb\log\log phpBB log*/
	protected $log;

	/** @var \phpbb\request\request */
	protected $request;

	/** @var \phpbb\template */
	protected $template;

	/** @var ContainerInterface */
	protected $phpbb_container;

	/** @var \phpbb\user */
	protected $user;

	/** @var string */
	protected $phpbb_root_path;
	protected $php_ext;

	/** @var string */
	public $u_action;

	public function main($id, $mode)
	{
		global $config, $db, $phpbb_log, $request, $template, $phpbb_container, $user, $phpbb_root_path, $phpEx;

		$this->config = $config;
		$this->db = $db;
		$this->log = $phpbb_log;
		$this->request = $request;
		$this->template = $template;
		$this->pagination = $phpbb_container->get('pagination');
		$this->user = $user;
		$this->root_path = $phpbb_root_path;
		$this->php_ext = $phpEx;

		// Start initial var setup
		$action = $this->request->variable('action', '');

		switch ($mode)
		{
			case 'settings':

				$display_vars = array(
					'title' => 'PM_SPY_SETTINGS',
				);

				$error = $this->pm_spy_settings();
			break;

			case 'pm_spy':

				$display_vars = array(
					'title' => 'ACP_PM_SPY',
				);

				$error = $this->pm_spy_list();
			break;

			default:
				trigger_error('NO_MODE', E_USER_ERROR);
			break;
		}

		$this->template->assign_vars(array(
			'MODE'			=> $mode,
			'ERROR_MSG'		=> implode('<br />', $error),
			'S_ERROR'		=> (sizeof($error)) ? true : false,
		));

		if ($action == 'whois')
		{
			include($this->root_path . 'includes/functions_user.' . $this->php_ext);

			$this->user->add_lang('acp/users');
			$this->page_title = 'WHOIS';
			$this->tpl_name = 'simple_body';

			$user_ip = $this->request->variable('user_ip', '');
			$domain = gethostbyaddr($user_ip);
			$ipwhois = user_ipwhois($user_ip);

			$this->template->assign_vars(array(
				'MESSAGE_TITLE'		=> sprintf($this->user->lang['IP_WHOIS_FOR'], $domain),
				'MESSAGE_TEXT'		=> nl2br($ipwhois))
			);

			return;
		}
		else
		{
			$this->tpl_name	= 'acp_pm_spy';
			$this->page_title = $this->user->lang[$display_vars['title']];
		}
	}

	public function pm_spy_settings()
	{
		$submit 	= ($this->request->is_set_post('submit')) ? true : false;
		$excl_groups	= ($this->config['pm_spy_excl_groups']) ? explode(',', $this->config['pm_spy_excl_groups']) : array();
		$error		= array();

		$form_key = 'acp_pm_spy_settings';
		add_form_key($form_key);

		if ($submit && !check_form_key($form_key))
		{
			$error[] = $this->user->lang['FORM_INVALID'];
		}

		// Do not write values if there is an error
		if (sizeof($error))
		{
			$submit = false;
		}

		if ($submit)
		{
			$this->config->set('pm_spy_excl_groups', implode(',', $this->request->variable('excl_groups', array(0))));

			trigger_error($this->user->lang['CONFIG_UPDATED'] . adm_back_link($this->u_action));
		}

		$sql = 'SELECT group_id, group_type, group_name
			FROM ' . GROUPS_TABLE;
		$result = $this->db->sql_query($sql);

		$exclusion_options = '';
		while ($row = $this->db->sql_fetchrow($result))
		{
			$group_name = ($row['group_type'] == GROUP_SPECIAL) ? $this->user->lang['G_' . $row['group_name'] . ''] : $row['group_name'];

			$selected = (in_array($row['group_id'], $excl_groups)) ? ' selected="selected"' : '';
			$exclusion_options .= '<option value="' . $row['group_id'] . '"' . $selected . ' style="font-weight: bold;">' . $group_name . '</option>';
		}
		$this->db->sql_freeresult($result);

		$this->template->assign_vars(array(
			'EXCLUSION_OPTIONS'		=> $exclusion_options,

			'U_ACTION'		=> $this->u_action,
		));

		return $error;
	}

	public function pm_spy_list()
	{
		$start		= $this->request->variable('start', 0);
		$sort_key	= $this->request->variable('sk', 'd');
		$sd = $sort_dir	= $this->request->variable('sd', 'd');
		$marked		= $this->request->variable('mark', array(''));
		$delete		= ($this->request->is_set_post('delete')) ? true : false;
		$per_page	= $this->config['topics_per_page'];
		$error		= array();

		if ($delete)
		{
			if (!sizeof($marked))
			{
				$error[] = $this->user->lang['NO_PM_SELECTED'];
			}
			else if (confirm_box(true))
			{
				// Restore the array to its correct format
				$marked = str_replace('#', '"', $marked);

				foreach ($marked as $pm_msg_list)
				{
					$pm_list[] = unserialize($pm_msg_list);
				}

				if (!function_exists('delete_pm'))
				{
					include($this->root_path . 'includes/functions_privmsgs.' . $this->php_ext);
				}

				foreach ($pm_list as $row)
				{
					delete_pm($row['user_id'], $row['msg_ids'], $row['folder_id']);
				}

				$this->log->add('admin', $this->user->data['user_id'], $this->user->ip, 'LOG_PM_SPY');
			}
			else
			{
				$s_hidden_fields = build_hidden_fields(array(
					'start'		=> $start,
					'delete'	=> $delete,
					'mark'		=> $marked,
					'sk'		=> $sort_key,
					'sd'		=> $sort_dir)
				);

				confirm_box(false, $this->user->lang['CONFIRM_OPERATION'], $s_hidden_fields, 'confirm_body.html');
			}
		}

		$sort_dir = ($sort_dir == 'd') ? ' DESC' : ' ASC';

		switch ($sort_key)
		{
			case 'b':
				$order_by = 'u.username_clean' . $sort_dir;
				$order_sql = ' AND t.user_id = u.user_id ';
			break;

			case 'd':
				$order_by = 'p.message_time' . $sort_dir;
				$order_sql = ' AND t.user_id = u.user_id ';
			break;

			case 'f':
				$order_by = 'u.username_clean' . $sort_dir;
				$order_sql = ' AND t.author_id = u.user_id ';
			break;

			case 'i':
				$order_by = 'p.author_ip' . $sort_dir. ', u.username_clean ASC';
				$order_sql = ' AND t.user_id = u.user_id ';
			break;

			case 'p':
				$order_by = 't.folder_id' . $sort_dir. ', u.username_clean ASC';
				$order_sql = ' AND t.user_id = u.user_id ';
			break;

			case 't':
				$order_by = 'to_username' . $sort_dir;
				$order_sql = ' AND t.user_id = u.user_id ';
			break;			
		}

		// Create a list of excluded users...
		$excluded = $excluded_p = '';

		$excluded_groups = ($this->config['pm_spy_excl_groups']) ? explode(',', $this->config['pm_spy_excl_groups']) : array();

		if (sizeof($excluded_groups))
		{
			$e_sql = 'SELECT user_id 
				FROM ' . USER_GROUP_TABLE . '
				WHERE ' . $this->db->sql_in_set('group_id', $excluded_groups);
			$e_result = $this->db->sql_query($e_sql);

			while ($e_row = $this->db->sql_fetchrow($e_result))
			{
				$excluded[] = $e_row['user_id'];
				$excluded_p[] = 'u_' . $e_row['user_id'];
			}
			$this->db->sql_freeresult($e_result);
		}

		$excl_sql = ($excluded) ? ' AND ' . $this->db->sql_in_set('t.user_id', $excluded, true) . '
			AND ' . $this->db->sql_in_set('t.author_id', $excluded, true) . '
			AND ' . $this->db->sql_in_set('p.to_address', $excluded_p, true) . '
			AND ' . $this->db->sql_in_set('p.bcc_address', $excluded_p, true) . '' : '';

		// Get PM count for pagination
		$sql = 'SELECT COUNT(t.msg_id) AS total_pm
			FROM ' . PRIVMSGS_TABLE . ' p, ' . PRIVMSGS_TO_TABLE . ' t
			WHERE p.msg_id = t.msg_id
				' . $excl_sql;
		$result = $this->db->sql_query($sql);
		$total_pms = (int) $this->db->sql_fetchfield('total_pm');
		$this->db->sql_freeresult($result);

		if ($start >= $total_pms)
		{
			$start = ($start - $per_page < 0) ? 0 : $start - $per_page;
		}

		$pm_box_ary = array(
			PRIVMSGS_HOLD_BOX	=> $this->user->lang['PM_HOLDBOX'],
			PRIVMSGS_NO_BOX		=> $this->user->lang['PM_NOBOX'],
			PRIVMSGS_OUTBOX		=> $this->user->lang['PM_OUTBOX'],
			PRIVMSGS_SENTBOX	=> $this->user->lang['PM_SENTBOX'],
			PRIVMSGS_INBOX		=> $this->user->lang['PM_INBOX'],
		);

		$flags = (($this->config['auth_bbcode_pm']) ? OPTION_FLAG_BBCODE : 0) + (($this->config['auth_smilies_pm']) ? OPTION_FLAG_SMILIES : 0) + (($this->config['allow_post_links']) ? OPTION_FLAG_LINKS : 0);

		$excl_sql = ($excluded) ? ' AND ' . $this->db->sql_in_set('p.author_id', $excluded, true) . '
			AND ' . $this->db->sql_in_set('t.user_id', $excluded, true) . '
			AND ' . $this->db->sql_in_set('t.author_id', $excluded, true) . '
			AND ' . $this->db->sql_in_set('p.to_address', $excluded_p, true) . '
			AND ' . $this->db->sql_in_set('p.bcc_address', $excluded_p, true) . '' : '';

		$sql = 'SELECT p.msg_id, p.message_subject, p.message_text, p.bbcode_uid, p.bbcode_bitfield, p.message_time, p.bcc_address, p.to_address, p.author_ip, t.user_id, t.author_id, t.folder_id, LOWER(u.username) AS to_username
			FROM ' . PRIVMSGS_TABLE . ' p, ' . PRIVMSGS_TO_TABLE . ' t, ' . USERS_TABLE . ' u
			WHERE p.msg_id = t.msg_id
				' . $order_sql . '
				' . $excl_sql . '
			ORDER BY ' . $order_by;
		$result = $this->db->sql_query_limit($sql, $per_page, $start);

		$i = 0;
		while ($row = $this->db->sql_fetchrow($result))
		{
			++$i;

			$this->template->assign_block_vars('pm_row', array(
				'AUTHOR_IP'			=> $row['author_ip'],
				'BCC'				=> ($row['bcc_address']) ? $this->get_pm_user_data($row['user_id'], $row['author_id']) : '',
				'DATE'				=> $this->user->format_date($row['message_time']),
				'FOLDER'			=> ($row['folder_id'] > PRIVMSGS_INBOX) ? $this->user->lang['PM_SAVED'] : $pm_box_ary[$row['folder_id']],
				'FROM'				=> $this->get_pm_user_data($row['author_id']),
				'IS_GROUP'			=> (strstr($row['to_address'], 'g')) ? 'G' : '',
				'U_WHOIS'			=> $this->u_action . "&amp;action=whois&amp;user_ip={$row['author_ip']}",

				'LAST_VISIT_FROM'	=> $this->get_last_visit($row['author_id']),
				'LAST_VISIT_TO'		=> ($row['to_address']) ? $this->get_last_visit($row['user_id'], $row['author_id']) : '',

				// We have to replace " in this variable because the template system will not parse it.
				'PM_ID'				=> str_replace('"', '#', serialize(array('msg_ids' => $row['msg_id'], 'user_id' => $row['user_id'], 'folder_id' => $row['folder_id']))),

				// Create a unique key for the js script
				'PM_KEY'			=> $i,
				'PM_SUBJECT'		=> $row['message_subject'],
				'PM_TEXT'			=> generate_text_for_display($row['message_text'], $row['bbcode_uid'], $row['bbcode_bitfield'], $flags),		
				'TO'				=> ($row['to_address']) ? $this->get_pm_user_data($row['user_id'], $row['author_id']) : '',
			));
		}
		$this->db->sql_freeresult($result);

		$sort_by_text = array('f' => $this->user->lang['SORT_FROM'], 't' => $this->user->lang['SORT_TO'], 'b' => $this->user->lang['SORT_BCC'], 'p' => $this->user->lang['SORT_PM_BOX'], 'i' => $this->user->lang['SORT_IP'], 'd' => $this->user->lang['SORT_DATE']);
		$limit_days = array();
		$s_sort_key = $s_limit_days = $s_sort_dir = $u_sort_param = '';
		gen_sort_selects($limit_days, $sort_by_text, $sort_days, $sort_key, $sd, $s_limit_days, $s_sort_key, $s_sort_dir, $u_sort_param);

		$this->pagination->generate_template_pagination($this->u_action . "&amp;sk=$sort_key&amp;sd=$sd", 'pagination', 'start', $total_pms, $per_page, $start);

		$this->template->assign_vars(array(
			'MESSAGE_COUNT'		=> $total_pms,

			'S_SORT_KEY'		=> $s_sort_key,
			'S_SORT_DIR'		=> $s_sort_dir,

			'U_ACTION'		=> $this->u_action . "&amp;sk=$sort_key&amp;sd=$sd&amp;start=$start",
		));

		return $error;
	}

	public function get_last_visit($user_id, $author = 0)
	{
		if ($user_id == $author)
		{
			$last_visit = '';
		}
		else
		{
			$sql = 'SELECT session_user_id, MAX(session_time) AS session_time
				FROM ' . SESSIONS_TABLE . '
				WHERE session_time >= ' . (time() - $this->config['session_length']) . '
					AND ' . $this->db->sql_in_set('session_user_id', $user_id) . '
				GROUP BY session_user_id';
			$result = $this->db->sql_query($sql);

			$session_times = array();
			while ($row = $this->db->sql_fetchrow($result))
			{
				$session_times[$row['session_user_id']] = $row['session_time'];
			}
			$this->db->sql_freeresult($result);

			$sql = 'SELECT user_lastvisit
				FROM ' . USERS_TABLE . '
				WHERE ' . $this->db->sql_in_set('user_id', $user_id);
			$result = $this->db->sql_query($sql);

			while ($row = $this->db->sql_fetchrow($result))
			{	
				$session_time = (!empty($session_times[$user_id])) ? $session_times[$user_id] : 0;
				$last_visit = (!empty($session_time)) ? $session_time : $row['user_lastvisit'];
				$last_visit = $this->user->format_date($last_visit);
			}
			$this->db->sql_freeresult($result);
		}

		return $last_visit;
	}

	public function get_pm_user_data($pm_user, $author = 0)
	{
		if ($pm_user == $author)
		{
			$user_info = '';
		}
		else
		{
			$sql = 'SELECT username, user_colour
				FROM ' . USERS_TABLE . '
				WHERE ' . $this->db->sql_in_set('user_id', $pm_user);
			$result = $this->db->sql_query($sql);
			$row = $this->db->sql_fetchrow($result);

			$user_info = get_username_string('full',(int) $pm_user, $row['username'], $row['user_colour']);
		}
		return $user_info;
	}
}
